--[[ Aftermath Thaddius Mod - Notifies players about power surge and 
     where to move during phase 2 of the Thaddius encounter.
     Author: Chop of Lightning's Blade(US)
     Contact: chop@aftermathlb.com ]]

-- constants
AMT_VER = "2.8";
AMT_POWERSURGE = "Stalagg gains Power Surge.";
AMT_POLARITYSHIFT = "Thaddius begins to cast Polarity Shift.";
AMT_PSYELL = "Now you feel pain...";
AMT_P1YELL1 = "Stalagg crush you!";
AMT_P1YELL2 = "Feed you to master!";
AMT_P2YELL1 = "Eat... your... bones...";
AMT_P2YELL2 = "Break... you!!";
AMT_P2YELL3 = "Kill...";
AMT_KILL = "Thaddius dies.";
AMT_TEXTURE_POS = "Interface\\Icons\\Spell_ChargePositive";
AMT_TEXTURE_NEG = "Interface\\Icons\\Spell_ChargeNegative";
AMT_EVENT_MOVE = 0;
AMT_EVENT_DISABLE = 1;
AMT_EVENT_RESET = 2;
AMT_EVENT_ENABLE = 3;
AMT_EVENT_SHIFTCHARGE = 4;
AMT_DELAY = 1.0;
AMT_FADE_DELAY = 10.0;
AMT_MAXLAG = 1.0;
AMT_RED = "|cFFFF0000";
AMT_BLUE = "|cFF0099FF";
AMT_GREEN = "|cFF00FF00";
AMT_END = "|r";

-- state vars
amthaddius = {enabled, oldcharge, newcharge, chargetime, voice, update_flag,
  uselag, autoenable, arrows, arrow_fade};

function AMThaddius_Output(dir)
  if (amthaddius.enabled) then
    if (dir == "left") then
      AMThaddiusMsgFrame:AddMessage("<<< MOVE LEFT <<<", 1.0, 0.0, 0.0);
      if (amthaddius.arrows) then
        AMThaddiusLeftFrame:Show();
      end
    elseif (dir == "right") then
      AMThaddiusMsgFrame:AddMessage(">>> MOVE RIGHT >>>", 1.0, 0.0, 0.0);
      if (amthaddius.arrows) then
        AMThaddiusRightFrame:Show();
      end
    end
    if (amthaddius.voice) then
      if (dir == "left") then
        PlaySoundFile("Interface\\Addons\\AMThaddius\\left.mp3");
      elseif (dir == "right") then
        PlaySoundFile("Interface\\Addons\\AMThaddius\\right.mp3");
      end
    else
      PlaySoundFile("Sound\\Doodad\\BellTollHorde.wav");
    end
    amthaddius.arrow_fade = dir;
  end
end

function AMThaddius_LoadVars()
  if (not amt_savedvars) then
    amt_savedvars = {autoenable, voice, uselag, arrows};
    amt_savedvars.autoenable = true;
    amt_savedvars.voice = true;
    amt_savedvars.uselag = true;
    amt_savedvars.arrows = true;
  else
    amthaddius.autoenable = amt_savedvars.autoenable;
    amthaddius.voice = amt_savedvars.voice;
    amthaddius.uselag = amt_savedvars.uselag;
    amthaddius.arrows = amt_savedvars.arrows;
  end
end

function AMThaddius_RegisterForSave()
  amt_savedvars.autoenable = amthaddius.autoenable;
  amt_savedvars.voice = amthaddius.voice;
  amt_savedvars.uselag = amthaddius.uselag;
  amt_savedvars.arrows = amthaddius.arrows;
end

function AMThaddius_SlashCmd(arg)
  local _, _, msg, args = string.find(arg, "(%w+)%s?(.*)");
  if (msg) then
    msg = strlower(msg);
  end
  if (msg == "help") then
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius slash help:");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt - Toggles mod on or off");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt reset - Resets internal state");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt voice - Toggles voices");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt arrows - Toggles arrows");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt lag - Toggles lag-awareness");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt autoenable - Toggles auto-enable");
    DEFAULT_CHAT_FRAME:AddMessage("  /amt config - Prints mod settings/version");
  elseif (msg == "arrows") then
    amthaddius.arrows = not amthaddius.arrows;
    local tmp;
    if (amthaddius.arrows) then
      tmp = AMT_GREEN.."on"..AMT_END;
    else
      tmp = AMT_RED.."off"..AMT_END;
    end
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Arrows "..tmp..".");
  elseif (msg == "autoenable") then
    amthaddius.autoenable = not amthaddius.autoenable;
    local tmp;
    if (amthaddius.autoenable) then
      tmp = AMT_GREEN.."on"..AMT_END;
    else
      tmp = AMT_RED.."off"..AMT_END;
    end
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Auto-enable "..tmp..".");
  elseif (msg == "voice") then
    amthaddius.voice = not amthaddius.voice;
    local tmp;
    if (amthaddius.voice) then
      tmp = AMT_GREEN.."on"..AMT_END;
    else
      tmp = AMT_RED.."off"..AMT_END;
    end
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Voices "..tmp..".");
  elseif (msg == "reset") then
    AMThaddius_EventHandler(AMT_EVENT_RESET);
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Mod reset.");
  elseif (msg == "lag") then
    amthaddius.uselag = not amthaddius.uselag;
    local tmp;
    if (amthaddius.uselag) then
      tmp = AMT_GREEN.."on"..AMT_END;
    else
      tmp = AMT_RED.."off"..AMT_END;
    end
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Lag-awareness "..tmp..".");
  elseif (msg == "config") then
    local tmp1, tmp2, tmp3, tmp4, tmp5;
    if (amthaddius.enabled) then
      tmp1 = AMT_GREEN.."enabled"..AMT_END;
    else
      tmp1 = AMT_RED.."disabled"..AMT_END;
    end
    if (amthaddius.voice) then
      tmp2 = AMT_GREEN.."enabled"..AMT_END;
    else
      tmp2 = AMT_RED.."disabled"..AMT_END;
    end
    if (amthaddius.uselag) then
      tmp3 = AMT_GREEN.."enabled"..AMT_END;
    else
      tmp3 = AMT_RED.."disabled"..AMT_END;
    end
    if (amthaddius.autoenable) then
      tmp4 = AMT_GREEN.."enabled"..AMT_END;
    else
      tmp4 = AMT_RED.."disabled"..AMT_END;
    end
    if (amthaddius.arrows) then
      tmp5 = AMT_GREEN.."enabled"..AMT_END;
    else
      tmp5 = AMT_RED.."disabled"..AMT_END;
    end
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: mod=["..tmp1.."] auto-enable=["..tmp4.."] voice=["..tmp2.."] arrows=["..tmp5.."] lag-aware=["..tmp3.."] version=["..AMT_GREEN..AMT_VER..AMT_END.."]");
  elseif (not msg) then
    amthaddius.enabled = not amthaddius.enabled;
    if (amthaddius.enabled) then
      AMThaddiusFrame:Show();
      DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Mod "..AMT_GREEN.."enabled"..AMT_END..".");
    else
      AMThaddiusFrame:Hide();
      DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Mod "..AMT_RED.."disabled"..AMT_END..".");
    end
  end
  AMThaddius_RegisterForSave();
end

function AMThaddius_OnLoad()
  amthaddius.enabled = false;
  amthaddius.oldcharge = nil;
  amthaddius.newcharge = nil;
  amthaddius.chargetime = 0;
  amthaddius.update_flag = false;
  amthaddius.arrow_fade = nil;
  amthaddius.voice = true;
  amthaddius.arrows = true;
  amthaddius.uselag = true;
  amthaddius.autoenable = true;

  this:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE");
  this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS");
  this:RegisterEvent("CHAT_MSG_MONSTER_YELL");
  this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
  this:RegisterEvent("VARIABLES_LOADED");

  SLASH_AMThaddius1 = "/amt";
  SlashCmdList["AMThaddius"] = AMThaddius_SlashCmd;

  AMThaddiusLeftFrame:SetScale(1.5);
  AMThaddiusRightFrame:SetScale(1.5);
  AMThaddiusLeftTexture:SetVertexColor(0,1,0);
  AMThaddiusRightTexture:SetVertexColor(0,1,0);
  AMThaddiusLeftFrame:Hide();
  AMThaddiusRightFrame:Hide();

  DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: "..AMT_VER.." loaded.");
end

function AMThaddius_OnEvent()
  if (amthaddius.enabled) then
    if (event == "CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE" and arg1 == AMT_POLARITYSHIFT) then
      AMThaddius_EventHandler(AMT_EVENT_SHIFTCHARGE);
    elseif (event == "CHAT_MSG_MONSTER_YELL" and arg1 == AMT_PSYELL) then
      amthaddius.update_flag = true;
      amthaddius.chargetime = GetTime();
    elseif (event == "CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS" and arg1 == AMT_POWERSURGE) then
      AMThaddiusMsgFrame:AddMessage("** STALAGG POWER SURGE - OVERHEAL **", 1.0, 0.0, 0.0);
      PlaySoundFile("Sound\\Doodad\\BellTollHorde.wav");
    elseif (event == "CHAT_MSG_COMBAT_HOSTILE_DEATH" and arg1 == AMT_KILL) then
      AMThaddius_EventHandler(AMT_EVENT_DISABLE);
    end
  end
  if (event == "CHAT_MSG_MONSTER_YELL") then
    if (arg1 == AMT_P1YELL1 or arg1 == AMT_P1YELL2 or arg1 == AMT_P2YELL1 or arg1 == AMT_P2YELL2 or arg1 == AMT_P2YELL3) then
      if (not amthaddius.enabled) then
        AMThaddius_EventHandler(AMT_EVENT_ENABLE);
      end
      AMThaddius_EventHandler(AMT_EVENT_RESET);
    end
  elseif (event == "VARIABLES_LOADED") then
    AMThaddius_LoadVars();
  end
end

function AMThaddius_EventHandler(event)
  if (event == AMT_EVENT_MOVE) then
    for i = 1, 16 do
      local texture, numapps, debufftype = UnitDebuff("player", i, 0);
      if (texture ~= nil and numapps == 0) then
        if (texture == AMT_TEXTURE_POS) then
          amthaddius.newcharge = "pos";
          break;
        elseif (texture == AMT_TEXTURE_NEG) then
          amthaddius.newcharge = "neg";
          break;
        end
      end
    end
    if (amthaddius.newcharge) then
      local tmp2;
      if (amthaddius.newcharge == "pos") then
        tmp2 = AMT_BLUE.."POS"..AMT_END;
      else
        tmp2 = AMT_RED.."NEG"..AMT_END;
      end
      if (amthaddius.oldcharge) then
        local tmp;
        if (amthaddius.oldcharge == "pos") then
          tmp = AMT_BLUE.."POS"..AMT_END;
        else
          tmp = AMT_RED.."NEG"..AMT_END;
        end
        DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: OldCharge=["..tmp.."] NewCharge=["..tmp2.."]");
      else
        DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: OldCharge=[NONE] NewCharge=["..tmp2.."]");
      end
      --[[ Algorithm:
           No Pre-existing charge: + go left, - go right
           New charge is same as old: go left
           New charge is diff than old: go right ]]
      if (amthaddius.oldcharge == nil) then
        -- No previous charge = Thaddius first engage
        if (amthaddius.newcharge == "pos") then
          AMThaddius_Output("left");
        elseif (amthaddius.newcharge == "neg") then
          AMThaddius_Output("right");
        end
      elseif (amthaddius.oldcharge == amthaddius.newcharge) then
        AMThaddius_Output("left");
      else
        AMThaddius_Output("right");
      end
    end
  elseif (event == AMT_EVENT_SHIFTCHARGE) then
    -- re-read charge just in case there was lag
    for i = 1, 16 do
      local texture, numapps, debufftype = UnitDebuff("player", i, 0);
      if (texture ~= nil and numapps == 0) then
        if (texture == AMT_TEXTURE_POS) then
          amthaddius.newcharge = "pos";
          break;
        elseif (texture == AMT_TEXTURE_NEG) then
          amthaddius.newcharge = "neg";
          break;
        end
      end
    end
    amthaddius.oldcharge = amthaddius.newcharge;
    AMThaddiusMsgFrame:AddMessage("** PREPARE TO MOVE **", 0.0, 0.6, 1.0);
  elseif (event == AMT_EVENT_DISABLE) then
    amthaddius.enabled = false;
    AMThaddiusFrame:Hide();
    AMThaddiusLeftFrame:Hide();
    AMThaddiusRightFrame:Hide();
    DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: Thaddius death detected, stopping mod.");
  elseif (event == AMT_EVENT_RESET) then
    amthaddius.oldcharge = nil;
    amthaddius.newcharge = nil;
    amthaddius.chargetime = 0;
    amthaddius.update_flag = false;
  elseif (event == AMT_EVENT_ENABLE) then
    if ((not amthaddius.enabled) and amthaddius.autoenable) then
      amthaddius.enabled = true;
      AMThaddiusFrame:Show();
      DEFAULT_CHAT_FRAME:AddMessage("AMThaddius: "..AMT_GREEN.."Auto-enabling"..AMT_END.." mod.");
    end
  end
end

function AMThaddius_OnUpdate(arg1)
  if (amthaddius.enabled) then
    if (amthaddius.update_flag) then
      local down, up, lag = GetNetStats();
      if (amthaddius.uselag) then
        lag = 2 * lag / 1000;
        if (lag > AMT_MAXLAG) then
          lag = AMT_MAXLAG;
        end
      else
        lag = 0;
      end
      if (GetTime() >= (amthaddius.chargetime + lag + AMT_DELAY)) then
        amthaddius.update_flag = false;
        AMThaddius_EventHandler(AMT_EVENT_MOVE);
      end
    end
    if (amthaddius.arrows and amthaddius.arrow_fade) then
      local tmp = nil;
      for i = 1, 16 do
        local texture, numapps, debufftype = UnitDebuff("player", i, 0);
        if (texture ~= nil and numapps == 0) then
          if (texture == AMT_TEXTURE_POS) then
            tmp = "pos";
            break;
          elseif (texture == AMT_TEXTURE_NEG) then
            tmp = "neg";
            break;
          end
        end
      end
      if (tmp and (tmp ~= amthaddius.newcharge)) then
        AMThaddiusMsgFrame:AddMessage("** LAGGED READ DETECTED: MOVE OPPOSITE DIRECTION IMMEDIATELY **", 0.0, 1.0, 0.0);
      end
      local x = GetTime();
      if (x < (amthaddius.chargetime + AMT_FADE_DELAY)) then
        if (math.mod(math.floor(x),2) == 0) then
          if (amthaddius.arrow_fade == "left") then
            AMThaddiusLeftFrame:Show();
          else
            AMThaddiusRightFrame:Show();
          end
        else
          if (amthaddius.arrow_fade == "left") then
            AMThaddiusLeftFrame:Hide();
          else
            AMThaddiusRightFrame:Hide();
          end
        end
      else
        amthaddius.arrow_fade = nil;
        AMThaddiusLeftFrame:Hide();
        AMThaddiusRightFrame:Hide();
      end
    end
  end
end

